#ifndef _ITI_TWI_H
#define _ITI_TWI_H

#include <inttypes.h>

////////////////////////////////////////////////////////////
// Init-, Start- und Stop-Routinen:
////////////////////////////////////////////////////////////

/* 	Setzt die Geschwindikeit des Busses abhngig von der 
	CPU-Geschwindigkeit. 
	  ==> SLCfreq==50KHz bei 1MHz CPU-Takt
	*/
void twi_init();


/*	Leitet einen TWI (I2C) Sende- oder Empfangsvorgang ein.
	Falls erfolgreich gibt 1 zurck, sonst 0; */
int twi_start();

int twi_rep_start();

/*	Beendet einen TWI (I2C) Sende- oder Empfangsvorgang. */
void twi_stop();


////////////////////////////////////////////////////////////
// Senderoutinen:

/* 	Alle Sende-Routinen geben eine 0 zurck,
	falls Sendevorgang nicht erfolgreich. 
	*/


/*  Sendet eine slave-Addresse, um anschlieend zu schreiben.
	(8Bit, LSB muss 0 sein) */
int twi_sla_w(uint8_t slave_adress);

/*  Sendet eine slave-Addresse, um anschlieend zu lesen.
	(8Bit, LSB muss 0 sein) */
int twi_sla_r(uint8_t slave_adress);

/*	Schreibt ein Byte auf den Bus */
int twi_send(uint8_t data);


////////////////////////////////////////////////////////////
// Empfangsroutine:


/*	Liest ein Byte vom Bus. 

	Parameter:
	int ack 		- ACK-Bit
	uint8_t* data	- Unter *data wird das gelesene Byte gespeichert

	Falls ack!=0 wird der Empfang mit einem	ACK-Bit besttigt 
	(== Erwarte nchstes Byte). Sonst wird das Bit nicht gesetzt 
	(== Transmission beendet).

	Rckgabewert:
	Die Methode gibt eine 0 zurck falls ein Fehler auftritt. Sonst eine 1.
	Ein Fehler tritt auf wenn:
		- data==0 (null-pointer)
		- Lesen nicht erfolgreich war.

	Falls das Lesen nicht erfolgreich war, wird *data auf 0x00 gesetzt.
	
*/
int twi_read(int ack, uint8_t* data);



#endif
