#include <avr/io.h>
#include <stdlib.h>
#include "lcd.h"
//#include <avrx-io.h>
//#include <avrx-signal.h>
//#include "avrx.h"
//#include "hardware.h"
#include <stdio.h>

//extern TimerControlBlock DisplayTimer;

char LCD_CTRL_RS     = (1<<1); // Register Select
char LCD_CTRL_RW     = (1<<4); // Read/Write
char LCD_CTRL_E      = (1<<0); // Bitmaske fr Enable

void lcd_ctrl(char c);
void lcd_wait();

void lcd_init() {
	DDRD |= LCD_CTRL_E|LCD_CTRL_RW|LCD_CTRL_RS; // als Ausgabepins konfigurieren (E, RW, RS)
	PORTD &= ~(LCD_CTRL_E|LCD_CTRL_RW|LCD_CTRL_RS);
	DDRB = 0xff; // Alle Pins des 8 Bit breiten LCD-Daten-Ports sind Ausgabepins.

	lcd_wait();
	lcd_ctrl(0b00001100); // Display einschalten.
	lcd_wait();
	lcd_ctrl(0b00111000); // Display auf 8-Bit-Ansteuerung und Zweizeilenbetrieb schalten.
}

// Low-Level-Funktion zum Schreiben eines Datenworts
// zum Displaycontroller.

void lcd_write( char c ) {
	if (c == LF) {
		lcd_wait();
		lcd_ctrl(0xc0);
	}
	else if (c == FF) {
		lcd_clear();
	}
	else if (c == VT) {
		lcd_wait();
		lcd_ctrl(0x80);
	}
	else { 
		lcd_putc(c);
	}
}	

void lcd_putc( char c ) {
	lcd_wait();
	char ctrl_port = PORTD;
	ctrl_port &= ~(LCD_CTRL_RW|LCD_CTRL_E);
	ctrl_port |= LCD_CTRL_RS;
	PORTD = ctrl_port; // Ausgeben: RS <- 1, R/W <- 0, E <- 0
	ctrl_port |= LCD_CTRL_E; // min. tAS (140 ns) warten
	PORTD = ctrl_port; // Ausgeben: E <- 1
	PORTB = c; // Ausgeben: D0 bis D7
	ctrl_port &= ~LCD_CTRL_E; // min. tDSW (195 ns) warten
	PORTD = ctrl_port; // Ausgeben: E <- 0
						// min. max{tAH,tH} (10 ns) warten
}

// Displayinhalt lschen.
void lcd_clear( void ) {
	lcd_wait();
	lcd_ctrl(0b00000001);
}

// Low-Level-Funktion zum Schreiben eines Kontrollworts
// zum Displaycontroller
void lcd_ctrl(char c)
{
	char ctrl_port = PORTD;
	ctrl_port &= ~(LCD_CTRL_RW|LCD_CTRL_RS|LCD_CTRL_E);
	PORTD = ctrl_port; // Ausgeben: RS <- 0, R/W <- 0, E <- 0

	ctrl_port |= LCD_CTRL_E; // min. tAS (140 ns) warten
	PORTD = ctrl_port; // Ausgeben: E <- 1
	PORTB = c; // Ausgeben: D0 bis D7
	ctrl_port &= ~LCD_CTRL_E; // min. tDSW (195 ns) warten
	PORTD = ctrl_port; // Ausgeben: E <- 0
 						// min. max{tAH,tH} (10 ns) warten
}


// Warten auf Ende der Verarbeitung im Displaycontroller,
// ermitteln der aktuellen Position in dessen Datenspeicher.
void lcd_wait() {

	PORTB = 0;
	DDRB = 0;

	char ctrl_port = PORTD;
	ctrl_port &= ~(LCD_CTRL_RS|LCD_CTRL_E); // RS <- 0, E <- 0
	ctrl_port |= LCD_CTRL_RW; // R/W <- 1
	PORTD = ctrl_port; // Ausgeben: E <- 0, R/W <- 1
	char data_port = 0;
	do {
		ctrl_port |= LCD_CTRL_E; // E <- 1, min. tAS (140 ns) warten
		PORTD = ctrl_port; // Ausgeben: E <- 1
		ctrl_port &= ~LCD_CTRL_E; // E <- 0, min. tDDR (320 ns) warten
		data_port = PINB; // Daten von Displaycontroller lesen
		PORTD = ctrl_port; // Ausgeben: E <- 0
	}
	while (data_port & 0b10000000); // Busy-Flag = 1?, dann warten.

	ctrl_port &= ~LCD_CTRL_RW;
	PORTD = ctrl_port; // Ausgeben: R/W <- 0
	DDRB = 0xff; // wieder als Ausgabe.

}


#define CLEN 10
void lcd_puti( int a ) {

	int i = 0;

	char s[CLEN];	
	s[0] = 0;

	// Schreibe den ganzzahligen Teil in den Buffer		
	itoa((int)a, s, 10);
		
	i = 0;
	while (s[i] && i < CLEN) {
		lcd_putc(s[i]); 
		i++;
	}

}



void lcd_putf( float a ) {
	int i = 0;
	int k = 0;
	float rest = 0.0;

	char s[CLEN];	
	s[0] = 0;

	// Schreibe den ganzzahligen Teil in den Buffer		
	itoa((int)a, s, 10);

	// Suche das Ende der ganzzahligen Teils und schreibe ein '.'
	k = 0;
	while (s[k]) {k++;}
	s[k] = '.'; k++;
	s[k] = 0;

	// Gebe den Nachkommateil aus:
	if (a < 0) (a=-a);
	rest = a-((int)a);
	itoa( (int)(rest*100), &(s[k]), 10);
		
	// Gebe die Zahl aus			
	i = 0;
	while (s[i] && i < CLEN) {
		lcd_putc(s[i]); 
		i++;
	}

}
